// Copyright (c) 2012 Cloudera, Inc.  All rights reserved.
define([
  "cloudera/cmf/parcel/Parcels",
  "cloudera/cmf/wizard/WizardStepBase",
  "knockout",
  "underscore"
], function (Parcels, WizardStepBase, ko, _) {
  "use strict";

  return WizardStepBase.extend({
    /**
     * options: {
     *   id:        (required) "the id of the step",
     *   updateUrl: (required) "the URL to fetch the parcel installation status",
     *   installUrl:(required) "the URL to install parcel",
     *   cluster:   (required) a function that returns the selected cluster",
     *   parcels:   (required) a function that returns the selected product/version",
     *   displayPageWhenActivated: (required) true means we can display the page even in activated mode.
     * }
     */
    init: function(options) {
      var self = this;
      self.options = options;
      // Keep a reference to this object
      // so we can change it.
      self.updateParams = {};
      self.showParcelContent = ko.observable(false);
      self.enableContinue = ko.observable(false);

      self.parcelsUpdater = new Parcels({
        container: "#" + options.id,
        updateUri: options.updateUrl,
        updateParams: self.updateParams,
        afterUpdate: _.bind(self.afterUpdate, self),
        separateRemote: false
      });
      self.viewModel = self.parcelsUpdater;
      self._super.apply(self, arguments);
    },

    afterUpdate: function() {
      var self = this,
        allActivated = true;

      _.each(self.viewModel.data(), function(clusterAndProduct) {
        _.each(clusterAndProduct.statesByProduct(), function(productAndStates) {
          _.each(productAndStates, function(productAndState) {
            var stage = productAndState.state.stage();
            if (stage === "DISTRIBUTING" || stage === "DOWNLOADING") {
              // once in progress, we can show the page content.
              // this is an one way street.
              // once the page is shown, we cannot go back.
              allActivated = false;
              self.showParcelContent(true);
            } else if (stage === "ACTIVATED") {
              if (self.options.displayPageWhenActivated) {
                self.showParcelContent(true);
              }
            } else {
              allActivated = false;
            }
          });
        });
      });

      // When we find all the stages are ACTIVATED, enable the Continue button.
      self.enableContinue(allActivated && self.showParcelContent());
    },

    _refresh: function(params) {
      var self = this;
      self.updateParams.clusterId = params.clusterId;
      self.updateParams.parcelsJson = params.parcelsJson;
      self.parcelsUpdater.start();
    },

    refresh: function() {
      var self = this;
      var hasNewParcels = !_.isEmpty(self.options.parcels());
      var params = {
        clusterId: self.options.cluster().id,
        parcelsJson: JSON.stringify(self.options.parcels())
      };

      if (hasNewParcels) {
        $.post(self.options.installUrl, params, function(response) {
          if (response.message === "OK") {
            // self.updateParams is an object that
            // we pass to self.parcelsUpdater.
            // so updating it also affects self.parcelsUpdater
            self._refresh(params);
          } else {
            $.publish("showError", [response.message]);
          }
        }, 'json');
      } else {
        self._refresh(params);
      }
    },

    beforeEnter: function(callback) {
      callback();
      this.refresh();
    }
  });
});
